@@ -77,7 +77,7 @@ import katex from "katex";
   var chemParse = function (tokens, stateMachine) {
     // Recreate the argument string from KaTeX's array of tokens.
     var str = "";
-    var expectedLoc = tokens[tokens.length - 1].loc.start
+    var expectedLoc = tokens.length && tokens[tokens.length - 1].loc.start
     for (var i = tokens.length - 1; i >= 0; i--) {
       if(tokens[i].loc.start > expectedLoc) {
         // context.consumeArgs has eaten a space.
