@@ -2,6 +2,7 @@
 "use strict";
 
 const childProcess = require("child_process");
+const util = require("util");
 const fs = require("fs-extra");
 const jspngopt = require("jspngopt");
 const net = require("net");
@@ -9,6 +10,9 @@ const os = require("os");
 const pako = require("pako");
 const path = require("path");
 const got = require("got");
+const pRetry = require('p-retry');
+
+const execFile = util.promisify(childProcess.execFile);
 
 const selenium = require("selenium-webdriver");
 const firefox = require("selenium-webdriver/firefox");
@@ -69,7 +73,8 @@ const opts = require("commander")
     .option("--browserstack", "Use Browserstack. The username and access key"
         + " should be set as enviroment variable BROWSERSTACK_USER and"
         + " BROWSERSTACK_ACCESS_KEY")
-    .parse(process.argv);
+    .parse(process.argv)
+    .opts();
 
 let listOfCases;
 if (opts.include) {
@@ -112,15 +117,6 @@ if (opts.browserstack) {
 //////////////////////////////////////////////////////////////////////
 // Work out connection to selenium docker container
 
-function check(err) {
-    if (!err) {
-        return;
-    }
-    console.error(err);
-    console.error(err.stack);
-    process.exit(1);
-}
-
 function cmd() {
     const args = Array.prototype.slice.call(arguments);
     const cmd = args.shift();
@@ -172,7 +168,8 @@ if (!seleniumURL && opts.container) {
         guessDockerIPs();
     }
     seleniumPort = cmd("docker", "port", opts.container, seleniumPort);
-    seleniumPort = seleniumPort.replace(/^.*:/, "");
+    // Docker can output two lines, such as "0.0.0.0:49156\n:::49156"
+    seleniumPort = seleniumPort.replace(/[^]*:([0-9]+)[^]*/, "$1");
 }
 if (!seleniumURL && seleniumIP) {
     seleniumURL = "http://" + seleniumIP + ":" + seleniumPort + "/wd/hub";
@@ -183,8 +180,36 @@ if (seleniumURL) {
     console.log("Selenium driver in local session");
 }
 
-process.nextTick(startServer);
-let attempts = 0;
+(async() => {
+    if (!(katexURL || katexPort)) {
+        await pRetry(startServer, {retries: 50, minTimeout: 100});
+    }
+    if (opts.seleniumProxy) {
+        driver = await getProxyDriver();
+    } else {
+        if (opts.browserstack) {
+            await startBrowserstackLocal();
+        }
+        if (seleniumIP) {
+            await pRetry(tryConnect, {retries: 50, minTimeout: 100});
+        }
+        driver = buildDriver();
+    }
+    await setupDriver();
+    await findHostIP();
+    await takeScreenshots();
+
+    await driver.quit();
+    await devServer.stop();
+    if (bsLocal) {
+        const bsLocalStop = util.promisify(bsLocal.stop);
+        await bsLocalStop();
+    }
+    process.exit(exitStatus);
+})().catch(err => {
+    console.error(err);
+    process.exit(1);
+});
 
 //////////////////////////////////////////////////////////////////////
 // Start up development server
@@ -194,13 +219,8 @@ let coverageMap;
 const minPort = 32768;
 const maxPort = 61000;
 
-function startServer() {
-    if (katexURL || katexPort) {
-        process.nextTick(tryConnect);
-        return;
-    }
-    const port = Math.floor(Math.random() * (maxPort - minPort)) + minPort;
-
+async function startServer() {
+    katexPort = Math.floor(Math.random() * (maxPort - minPort)) + minPort;
     if (opts.coverage) {
         coverageMap = istanbulLibCoverage.createCoverageMap({});
         webpackConfig.module.rules[0].use = {
@@ -215,69 +235,50 @@ function startServer() {
     }
     const config = {
         ...webpackConfig.devServer,
-        port,
+        static: [{directory: process.cwd(), watch: false}],
+        port: katexPort,
         hot: false,
         liveReload: false,
-        injectClient: false,
+        client: false,
     };
     const compiler = webpack(webpackConfig);
-    const wds = new WebpackDevServer(compiler, config);
-    wds.listen(port).then(server => {
-        server.once("listening", function() {
-            devServer = wds;
-            katexPort = port;
-            attempts = 0;
-            process.nextTick(opts.seleniumProxy ? getProxyDriver
-                : opts.browserstack ? startBrowserstackLocal : tryConnect);
-        });
-        server.on("error", function(err) {
-            if (devServer !== null) { // error after we started listening
-                throw err;
-            } else if (++attempts > 50) {
-                throw new Error("Failed to start up dev server");
-            } else {
-                process.nextTick(startServer);
-            }
-        });
-    });
+    devServer = new WebpackDevServer(config, compiler);
+    await devServer.start(katexPort);
 }
 
 // Start Browserstack Local connection
-function startBrowserstackLocal() {
+async function startBrowserstackLocal() {
     // unique identifier for the session
     const localIdentifier = process.env.CIRCLE_BUILD_NUM || "p" + katexPort;
     opts.seleniumCapabilities["browserstack.localIdentifier"] = localIdentifier;
 
     bsLocal = new browserstack.Local();
-    bsLocal.start({localIdentifier}, function(err) {
-        if (err) {
-            throw err;
-        }
-        process.nextTick(tryConnect);
+    await new Promise((resolve, reject) => {
+        bsLocal.start({localIdentifier}, err => {
+            if (err) {
+                reject(err);
+            } else {
+                resolve();
+            }
+        });
     });
 }
 
 //////////////////////////////////////////////////////////////////////
 // Wait for container to become ready
 
-function tryConnect() {
-    if (!seleniumIP) {
-        process.nextTick(buildDriver);
-        return;
-    }
-    const sock = net.connect({
-        host: seleniumIP,
-        port: +seleniumPort,
-    });
-    sock.on("connect", function() {
-        sock.end();
-        attempts = 0;
-        process.nextTick(buildDriver);
-    }).on("error", function() {
-        if (++attempts > 50) {
-            throw new Error("Failed to connect selenium server.");
-        }
-        setTimeout(tryConnect, 200);
+async function tryConnect() {
+    return new Promise((resolve, reject) => {
+        const sock = net.connect({
+            host: seleniumIP,
+            port: +seleniumPort,
+        });
+        sock.on("connect", function() {
+            sock.end();
+            resolve();
+        }).on("error", function(err) {
+            reject(err);
+        });
     });
 }
 
@@ -289,11 +290,10 @@ let driverReady = false;
 function buildDriver() {
     const builder = new selenium.Builder().forBrowser(opts.browser);
     if (opts.browser === "firefox") {
-        const ffProfile = new firefox.Profile();
-        ffProfile.setPreference(
+        const ffOptions = new firefox.Options();
+        ffOptions.setPreference(
             "browser.startup.homepage_override.mstone", "ignore");
-        ffProfile.setPreference("browser.startup.page", 0);
-        const ffOptions = new firefox.Options().setProfile(ffProfile);
+        ffOptions.setPreference("browser.startup.page", 0);
         builder.setFirefoxOptions(ffOptions);
     } else if (opts.browser === "chrome") {
         // https://stackoverflow.com/questions/48450594/selenium-timed-out-receiving-message-from-renderer
@@ -304,41 +304,38 @@ function buildDriver() {
         builder.usingServer(seleniumURL);
     }
     if (opts.seleniumCapabilities) {
+        // TODO: withCapabilities is deprecated
         builder.withCapabilities(opts.seleniumCapabilities);
     }
-    driver = builder.build();
-    setupDriver();
+    return builder.build();
 }
 
-function getProxyDriver() {
-    got.post(opts.seleniumProxy, {
+async function getProxyDriver() {
+    const {body} = await got.post(opts.seleniumProxy, {
         json: {
             browserstack: opts.browserstack,
             capabilities: opts.seleniumCapabilities,
             seleniumURL,
         },
         responseType: 'json',
-    }).then(({body}) => {
-        const session = new selenium.Session(body.id, body.capabilities);
-        const client = Promise.resolve(seleniumURL)
-            .then(url => new seleniumHttp.HttpClient(url));
-        const executor = new seleniumHttp.Executor(client);
-        driver = new selenium.WebDriver(session, executor);
-        setupDriver();
     });
+    const session = new selenium.Session(body.id, body.capabilities);
+    const client = Promise.resolve(new seleniumHttp.HttpClient(seleniumURL));
+    const executor = new seleniumHttp.Executor(client);
+    return new selenium.WebDriver(session, executor);
 }
 
-function setupDriver() {
-    driver.manage().timeouts().setScriptTimeout(3000).then(function() {
-        let html = '<!DOCTYPE html>' +
-            '<html><head><style type="text/css">html,body{' +
-            'width:100%;height:100%;margin:0;padding:0;overflow:hidden;' +
-            '}</style></head><body><p>Test</p></body></html>';
-        html = "data:text/html," + encodeURIComponent(html);
-        return driver.get(html);
-    }).then(function() {
-        setSize(targetW, targetH);
-    });
+async function setupDriver() {
+    await driver.manage().setTimeouts({script: 5000});
+
+    let html = '<!DOCTYPE html>' +
+        '<html><head><style type="text/css">html,body{' +
+        'width:100%;height:100%;margin:0;padding:0;overflow:hidden;' +
+        '}</style></head><body><p>Test</p></body></html>';
+    html = "data:text/html," + encodeURIComponent(html);
+    await driver.get(html);
+
+    await setSize(targetW, targetH);
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -346,22 +343,20 @@ function setupDriver() {
 
 const targetW = 1024;
 const targetH = 768;
-function setSize(reqW, reqH) {
-    return driver.manage().window().setSize(reqW, reqH).then(function() {
-        return driver.takeScreenshot();
-    }).then(function(img) {
-        img = imageDimensions(img);
-        const actualW = img.width;
-        const actualH = img.height;
-        if (actualW === targetW && actualH === targetH) {
-            findHostIP();
-            return;
-        }
-        if (++attempts > opts.attempts) {
-            throw new Error("Failed to set window size correctly.");
-        }
-        return setSize(targetW + reqW - actualW, targetH + reqH - actualH);
-    }, check);
+let attempts = 0;
+async function setSize(width, height) {
+    await driver.manage().window().setRect({width, height});
+    let img = await driver.takeScreenshot();
+    img = imageDimensions(img);
+    const actualW = img.width;
+    const actualH = img.height;
+    if (actualW === targetW && actualH === targetH) {
+        return;
+    }
+    if (++attempts > opts.attempts) {
+        throw new Error("Failed to set window size correctly.");
+    }
+    return setSize(targetW + width - actualW, targetH + height - actualH);
 }
 
 function imageDimensions(img) {
@@ -376,7 +371,7 @@ function imageDimensions(img) {
 //////////////////////////////////////////////////////////////////////
 // Work out how to connect to host KaTeX server
 
-function findHostIP() {
+async function findHostIP() {
     if (!katexIP) {
         katexIP = "localhost";
     }
@@ -385,21 +380,22 @@ function findHostIP() {
             katexURL = "http://" + katexIP + ":" + katexPort + "/";
             console.log("KaTeX URL is " + katexURL);
         }
-        process.nextTick(takeScreenshots);
         return;
     }
 
     // Now we need to find an IP the container can connect to.
     // First, install a server component to get notified of successful connects
-    devServer.app.get("/ss-connect.js", function(req, res, next) {
-        if (!katexURL) {
-            katexIP = req.query.ip;
-            katexURL = "http://" + katexIP + ":" + katexPort + "/";
-            console.log("KaTeX URL is " + katexURL);
-            process.nextTick(takeScreenshots);
-        }
-        res.setHeader("Content-Type", "text/javascript");
-        res.send("//OK");
+    const connect = new Promise((resolve) => {
+        devServer.app.get("/ss-connect.js", function(req, res, next) {
+            if (!katexURL) {
+                katexIP = req.query.ip;
+                katexURL = "http://" + katexIP + ":" + katexPort + "/";
+                console.log("KaTeX URL is " + katexURL);
+            }
+            res.setHeader("Content-Type", "text/javascript");
+            res.send("//OK");
+            resolve();
+        });
     });
 
     // Next, enumerate all network addresses
@@ -428,22 +424,41 @@ function findHostIP() {
     }).join("\n");
     html += "\n</body></html>";
     html = "data:text/html," + encodeURIComponent(html);
-    driver.get(html);
+    await driver.get(html);
+    await connect;
 }
 
 //////////////////////////////////////////////////////////////////////
 // Take the screenshots
 
-let countdown = listOfCases.length;
-
 let exitStatus = 0;
 const listOfFailed = [];
 
-function takeScreenshots() {
-    listOfCases.forEach(takeScreenshot);
+async function takeScreenshots() {
+    for (const key of listOfCases) {
+        await takeScreenshot(key);
+    }
+
+    if (listOfFailed.length) {
+        console.error("Failed: " + listOfFailed.join(" "));
+    }
+    if (opts.diff) {
+        console.log("Diffs have been generated in: " + diffDir);
+    }
+    if (opts.new) {
+        console.log("New screenshots have been generated in: " + newDir);
+    }
+    if (opts.coverage) {
+        await collectCoverage();
+        const context = istanbulLibReport.createContext({coverageMap});
+        ['json', 'text', 'lcov'].forEach(fmt => {
+            const report = istanbulReports.create(fmt);
+            report.execute(context);
+        });
+    }
 }
 
-function takeScreenshot(key) {
+async function takeScreenshot(key) {
     const itm = data[key];
     if (!itm) {
         console.error("Test case " + key + " not known!");
@@ -451,70 +466,47 @@ function takeScreenshot(key) {
         if (exitStatus === 0) {
             exitStatus = 1;
         }
-        oneDone();
         return;
     }
 
     let file = path.join(dstDir, key + "-" + opts.browser + ".png");
     let retry = 0;
-    let loadExpected = null;
-    if (opts.verify) {
-        loadExpected = fs.readFile(file);
+    let expected = null;
+    if (opts.verify && await fs.pathExists(file)) {
+        expected = await fs.readFile(file);
     }
-
     const url = katexURL + "test/screenshotter/test.html?" + itm.query;
-    driver.call(loadMath);
+    let buf;
 
-    function loadMath() {
+    while (++retry <= opts.attempts) {
         if (!opts.reload && driverReady) {
-            driver.executeScript(
+            await driver.executeScript(
                     "handle_search_string(" +
-                    JSON.stringify("?" + itm.query) + ");")
-                .then(waitThenScreenshot);
-        } else if (opts.coverage) {
-            // collect coverage before reloading
-            collectCoverage().then(function() {
-                return driver.get(url).then(loadFonts);
-            });
+                    JSON.stringify("?" + itm.query) + ");");
         } else {
-            driver.get(url).then(loadFonts);
+            if (opts.coverage) {
+                // collect coverage before reloading
+                await collectCoverage();
+            }
+            await driver.get(url);
+            try {
+                await driver.executeAsyncScript(
+                        "var callback = arguments[arguments.length - 1]; " +
+                        "load_fonts_and_images(callback);");
+            } catch (e) {
+                console.error(e);
+            }
+            driverReady = true;
         }
-    }
-
-    function collectCoverage() {
-        return driver.executeScript('return window.__coverage__;')
-            .then(function(result) {
-                if (result) {
-                    coverageMap.merge(result);
-                }
-            });
-    }
-
-    function loadFonts() {
-        driver.executeAsyncScript(
-                "var callback = arguments[arguments.length - 1]; " +
-                "load_fonts(callback);")
-            .then(waitThenScreenshot);
-    }
-
-    function waitThenScreenshot() {
-        driverReady = true;
         if (opts.wait) {
-            browserSideWait(1000 * opts.wait);
+            await browserSideWait(1000 * opts.wait);
         }
-        const promise = driver.takeScreenshot().then(haveScreenshot);
-        if (retry === 0) {
-            // The `oneDone` promise remains outstanding if we retry, so
-            // don't re-add it
-            promise.then(oneDone, check);
-        }
-    }
-
-    function haveScreenshot(img) {
+        let img = await driver.takeScreenshot();
         img = imageDimensions(img);
         if (img.width !== targetW || img.height !== targetH) {
-            throw new Error("Expected " + targetW + " x " + targetH +
-                            ", got " + img.width + "x" + img.height);
+            console.error("Expected " + targetW + " x " + targetH +
+                          ", got " + img.width + "x" + img.height);
+            await setSize(targetW, targetH);
         }
         if (key === "Lap" && opts.browser === "firefox" &&
             img.buf[0x32] === 0xf8) {
@@ -526,141 +518,77 @@ function takeScreenshot(key) {
              */
             key += "_alt";
             file = path.join(dstDir, key + "-" + opts.browser + ".png");
-            if (loadExpected) {
-                loadExpected = fs.readFile(file);
+            if (expected) {
+                expected = await fs.readFile(file);
             }
         }
         const opt = new jspngopt.Optimizer({
             pako: pako,
         });
-        const buf = opt.bufferSync(img.buf);
-        if (loadExpected) {
-            return loadExpected.then(function(expected) {
-                if (!buf.equals(expected)) {
-                    if (++retry >= opts.attempts) {
-                        console.error("FAIL! " + key);
-                        listOfFailed.push(key);
-                        exitStatus = 3;
-                        if (opts.diff || opts.new) {
-                            return saveFailedScreenshot(key, buf);
-                        }
-                    } else {
-                        console.log("error " + key);
-                        browserSideWait(300 * retry);
-                        if (retry > 1) {
-                            driverReady = false; // reload fully
-                        }
-                        return driver.call(loadMath);
-                    }
-                } else {
-                    console.log("* ok  " + key);
-                }
-            });
+        buf = opt.bufferSync(img.buf);
+        if (expected) {
+            if (buf.equals(expected)) {
+                console.log("* ok  " + key);
+                return;
+            }
+            console.log("error " + key);
+            await browserSideWait(300 * retry);
+            if (retry > 1) {
+                driverReady = false; // reload fully
+            }
         } else {
-            return fs.writeFile(file, buf).then(function() {
-                console.log(key);
-            });
+            await fs.writeFile(file, buf);
+            console.log(key);
+            return;
         }
     }
 
-    function saveFailedScreenshot(key, buf) {
+    console.error("FAIL! " + key);
+    listOfFailed.push(key);
+    exitStatus = 3;
+    if (opts.diff || opts.new) {
         const filenamePrefix = key + "-" + opts.browser;
         const outputDir = opts.new ? newDir : diffDir;
         const baseFile = path.join(dstDir, filenamePrefix + ".png");
         const diffFile = path.join(diffDir, filenamePrefix + "-diff.png");
         const bufFile = path.join(outputDir, filenamePrefix + ".png");
 
-        let promise = fs.ensureDir(outputDir)
-            .then(function() {
-                return fs.writeFile(bufFile, buf);
-            });
+        await fs.ensureDir(outputDir);
+        await fs.writeFile(bufFile, buf);
+
         if (opts.diff) {
-            promise = promise.then(fs.ensureDir(diffDir))
-                .then(function() {
-                    return execFile("convert", [
-                        "-fill", "white",
-                        // First image: saved screenshot in red
-                        "(", baseFile, "-colorize", "100,0,0", ")",
-                        // Second image: new screenshot in green
-                        "(", bufFile, "-colorize", "0,80,0", ")",
-                        // Composite them
-                        "-compose", "darken", "-composite",
-                        "-trim",  // remove everything with the same color as
-                                  // the corners
-                        diffFile, // output file name
-                    ]);
-                });
+            await fs.ensureDir(diffDir);
+            await execFile("convert", [
+                "-fill", "white",
+                // First image: saved screenshot in red
+                "(", baseFile, "-colorize", "100,0,0", ")",
+                // Second image: new screenshot in green
+                "(", bufFile, "-colorize", "0,80,0", ")",
+                // Composite them
+                "-compose", "darken", "-composite",
+                "-trim",  // remove everything with the same color as
+                            // the corners
+                diffFile, // output file name
+            ]);
         }
         if (!opts.new) {
-            promise = promise.then(function() {
-                return fs.unlink(bufFile);
-            });
+            await fs.unlink(bufFile);
         }
-        return promise;
-    }
-
-    function oneDone() {
-        if (--countdown === 0) {
-            if (listOfFailed.length) {
-                console.error("Failed: " + listOfFailed.join(" "));
-            }
-            if (opts.diff) {
-                console.log("Diffs have been generated in: " + diffDir);
-            }
-            if (opts.new) {
-                console.log("New screenshots have been generated in: " + newDir);
-            }
-            if (opts.coverage) {
-                collectCoverage().then(function() {
-                    const context = istanbulLibReport.createContext({coverageMap});
-                    ['json', 'text', 'lcov'].forEach(fmt => {
-                        const report = istanbulReports.create(fmt);
-                        report.execute(context);
-                    });
-                    done();
-                });
-                return;
-            }
-            done();
-        }
-    }
-
-    function done() {
-        // devServer.close(cb) will take too long.
-        driver.quit().then(() => {
-            if (bsLocal) {
-                bsLocal.stop(() => {
-                    process.exit(exitStatus);
-                });
-            } else {
-                process.exit(exitStatus);
-            }
-        });
     }
 }
 
 // Wait using a timeout call in the browser, to ensure that the wait
 // time doesn't start before the page has reportedly been loaded.
-function browserSideWait(milliseconds) {
+async function browserSideWait(milliseconds) {
     // The last argument (arguments[1] here) is the callback to selenium
-    return driver.executeAsyncScript(
+    await driver.executeAsyncScript(
         "window.setTimeout(arguments[1], arguments[0]);",
         milliseconds);
 }
 
-// Execute a given command, and return a promise to its output.
-function execFile(cmd, args, opts) {
-    return new Promise(function(resolve, reject) {
-        childProcess.execFile(cmd, args, opts, function(err, stdout, stderr) {
-            if (err) {
-                console.error("Error executing " + cmd + " " + args.join(" "));
-                console.error(stdout + stderr);
-                err.stdout = stdout;
-                err.stderr = stderr;
-                reject(err);
-            } else {
-                resolve(stdout);
-            }
-        });
-    });
+async function collectCoverage() {
+    const result = await driver.executeScript('return window.__coverage__;');
+    if (result) {
+        coverageMap.merge(result);
+    }
 }
